/*
* This file is part of the LibreOffice project.
*
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this
* file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

#include "Timer.h"

int Timer::flag = 0;

Timer::Timer()
{
    if(flag == 0)
    {
        initTimer();
    }
    flag++;
}

TraceTime ::TraceTime(string name, int type)
{
    funcName = name;
    funcType = type;
    if (funcType == 0)
    {
#ifdef _WIN32
        _LARGE_INTEGER tmp;
        QueryPerformanceCounter(&tmp);
        cpustartTime.insert(std::pair<std::string, double>(funcName,
            (double)tmp.QuadPart));
#else
        double startTime;
#ifdef _linux_
        gettimeofday(&startTime, NULL);
        cpustartTime.insert(std::pair<std::string, double>(funcName,
            startTime));
#else
        startTime = clock();
        cpustartTime.insert(std::pair<std::string, double>(funcName,
            startTime));
#endif
#endif
    }
    else if (funcType == 1)
    {
        glGenQueries(2, queryID);
        glQueryCounter(queryID[0], GL_TIMESTAMP);
        glGetQueryObjectui64v(queryID[0], GL_QUERY_RESULT, &startTime);
        glstartTime.insert(std::pair<std::string, double>(funcName,
            (double)startTime));
    }
}

TraceTime::~TraceTime()
{
    if (funcType == 0)
    {
#ifdef _WIN32
        _LARGE_INTEGER tmp;
        QueryPerformanceCounter(&tmp);
        cpuendTime.insert(std::pair<std::string, double>(funcName,
            (double)tmp.QuadPart));

#else
        double startTime;
#ifdef _linux_
        gettimeofday(&startTime, NULL);
        cpuendTime.insert(std::pair<std::string, double>(funcName, startTime));
#else
        startTime = clock();
        cpuendTime.insert(std::pair<std::string, double>(funcName, startTime));
#endif
#endif
    }
    else if (funcType == 1)
    {
        GLuint64 stopTime;
        glQueryCounter(queryID[1], GL_TIMESTAMP);
        GLint stopTimerAvailable = 0;
        while (!stopTimerAvailable) {
            glGetQueryObjectiv(queryID[1], 
                GL_QUERY_RESULT_AVAILABLE, 
                &stopTimerAvailable);
        }
        glGetQueryObjectui64v(queryID[1], GL_QUERY_RESULT, &stopTime);
        glendTime.insert(std::pair<std::string, double>(funcName,
            (double)stopTime));
    }

}
Timer::~Timer()
{
    Time::iterator i;
    Time::iterator j;
    Time::iterator k;
    Time::iterator l;
    for (i = cpustartTime.begin(); i != cpustartTime.end();)
    {
        double eTime = 0;
        std::pair <Time::iterator, Time::iterator> ret;
        std::pair <Time::iterator, Time::iterator> retEnd;
        j = cpuendTime.find(i->first);
        ret = cpustartTime.equal_range(i->first);
        retEnd = cpuendTime.equal_range(j->first);
        for (k = ret.first, l = retEnd.first;
             k != ret.second, l != retEnd.second; k++, l++)
        {
            eTime +=  ((((l->second) - (k->second)) / dqFreq) * 1000); 
            i++;
            j++;
        }
#if WRITETIME2FILE
        timerFile << "Total Time " << std::fixed << (--k)->first << " : "
            << eTime << "ms" << endl;
#endif
    }

    for (i = glstartTime.begin(); i != glstartTime.end();)
    {
        double eTime = 0;
        std::pair <Time::iterator, Time::iterator> ret;
        std::pair <Time::iterator, Time::iterator> retEnd;
        j = glendTime.find(i->first);
        ret = glstartTime.equal_range(i->first);
        retEnd = glendTime.equal_range(j->first);
        for (k = ret.first, l = retEnd.first;
             k != ret.second, l != retEnd.second; k++, l++)
        {
            eTime +=  (((l->second) - (k->second)) / 1000000); 
            i++;
            j++;
        }
#if WRITETIME2FILE
        timerFile << "Total Time " << std::fixed << (--k)->first << " : "
            << eTime << "ms" << endl;
#endif
    }
}

void Timer::initTimer()
{
#ifdef _WIN32
    QueryPerformanceCounter(&temp);
    dqFreq = (double)temp.QuadPart;
#endif
#if WRITETIME2FILE
    timerFile = ofstream(TIMER_FILE);
#endif
}
